// Code generated by ent, DO NOT EDIT.

package ent

import (
	"context"
	"errors"
	"fmt"

	"entgo.io/ent/dialect/sql"
	"entgo.io/ent/dialect/sql/sqlgraph"
	"entgo.io/ent/schema/field"
	"github.com/testifysec/archivista/ent/attestation"
	"github.com/testifysec/archivista/ent/attestationcollection"
	"github.com/testifysec/archivista/ent/predicate"
)

// AttestationUpdate is the builder for updating Attestation entities.
type AttestationUpdate struct {
	config
	hooks    []Hook
	mutation *AttestationMutation
}

// Where appends a list predicates to the AttestationUpdate builder.
func (au *AttestationUpdate) Where(ps ...predicate.Attestation) *AttestationUpdate {
	au.mutation.Where(ps...)
	return au
}

// SetType sets the "type" field.
func (au *AttestationUpdate) SetType(s string) *AttestationUpdate {
	au.mutation.SetType(s)
	return au
}

// SetAttestationCollectionID sets the "attestation_collection" edge to the AttestationCollection entity by ID.
func (au *AttestationUpdate) SetAttestationCollectionID(id int) *AttestationUpdate {
	au.mutation.SetAttestationCollectionID(id)
	return au
}

// SetAttestationCollection sets the "attestation_collection" edge to the AttestationCollection entity.
func (au *AttestationUpdate) SetAttestationCollection(a *AttestationCollection) *AttestationUpdate {
	return au.SetAttestationCollectionID(a.ID)
}

// Mutation returns the AttestationMutation object of the builder.
func (au *AttestationUpdate) Mutation() *AttestationMutation {
	return au.mutation
}

// ClearAttestationCollection clears the "attestation_collection" edge to the AttestationCollection entity.
func (au *AttestationUpdate) ClearAttestationCollection() *AttestationUpdate {
	au.mutation.ClearAttestationCollection()
	return au
}

// Save executes the query and returns the number of nodes affected by the update operation.
func (au *AttestationUpdate) Save(ctx context.Context) (int, error) {
	var (
		err      error
		affected int
	)
	if len(au.hooks) == 0 {
		if err = au.check(); err != nil {
			return 0, err
		}
		affected, err = au.sqlSave(ctx)
	} else {
		var mut Mutator = MutateFunc(func(ctx context.Context, m Mutation) (Value, error) {
			mutation, ok := m.(*AttestationMutation)
			if !ok {
				return nil, fmt.Errorf("unexpected mutation type %T", m)
			}
			if err = au.check(); err != nil {
				return 0, err
			}
			au.mutation = mutation
			affected, err = au.sqlSave(ctx)
			mutation.done = true
			return affected, err
		})
		for i := len(au.hooks) - 1; i >= 0; i-- {
			if au.hooks[i] == nil {
				return 0, fmt.Errorf("ent: uninitialized hook (forgotten import ent/runtime?)")
			}
			mut = au.hooks[i](mut)
		}
		if _, err := mut.Mutate(ctx, au.mutation); err != nil {
			return 0, err
		}
	}
	return affected, err
}

// SaveX is like Save, but panics if an error occurs.
func (au *AttestationUpdate) SaveX(ctx context.Context) int {
	affected, err := au.Save(ctx)
	if err != nil {
		panic(err)
	}
	return affected
}

// Exec executes the query.
func (au *AttestationUpdate) Exec(ctx context.Context) error {
	_, err := au.Save(ctx)
	return err
}

// ExecX is like Exec, but panics if an error occurs.
func (au *AttestationUpdate) ExecX(ctx context.Context) {
	if err := au.Exec(ctx); err != nil {
		panic(err)
	}
}

// check runs all checks and user-defined validators on the builder.
func (au *AttestationUpdate) check() error {
	if v, ok := au.mutation.GetType(); ok {
		if err := attestation.TypeValidator(v); err != nil {
			return &ValidationError{Name: "type", err: fmt.Errorf(`ent: validator failed for field "Attestation.type": %w`, err)}
		}
	}
	if _, ok := au.mutation.AttestationCollectionID(); au.mutation.AttestationCollectionCleared() && !ok {
		return errors.New(`ent: clearing a required unique edge "Attestation.attestation_collection"`)
	}
	return nil
}

func (au *AttestationUpdate) sqlSave(ctx context.Context) (n int, err error) {
	_spec := &sqlgraph.UpdateSpec{
		Node: &sqlgraph.NodeSpec{
			Table:   attestation.Table,
			Columns: attestation.Columns,
			ID: &sqlgraph.FieldSpec{
				Type:   field.TypeInt,
				Column: attestation.FieldID,
			},
		},
	}
	if ps := au.mutation.predicates; len(ps) > 0 {
		_spec.Predicate = func(selector *sql.Selector) {
			for i := range ps {
				ps[i](selector)
			}
		}
	}
	if value, ok := au.mutation.GetType(); ok {
		_spec.Fields.Set = append(_spec.Fields.Set, &sqlgraph.FieldSpec{
			Type:   field.TypeString,
			Value:  value,
			Column: attestation.FieldType,
		})
	}
	if au.mutation.AttestationCollectionCleared() {
		edge := &sqlgraph.EdgeSpec{
			Rel:     sqlgraph.M2O,
			Inverse: true,
			Table:   attestation.AttestationCollectionTable,
			Columns: []string{attestation.AttestationCollectionColumn},
			Bidi:    false,
			Target: &sqlgraph.EdgeTarget{
				IDSpec: &sqlgraph.FieldSpec{
					Type:   field.TypeInt,
					Column: attestationcollection.FieldID,
				},
			},
		}
		_spec.Edges.Clear = append(_spec.Edges.Clear, edge)
	}
	if nodes := au.mutation.AttestationCollectionIDs(); len(nodes) > 0 {
		edge := &sqlgraph.EdgeSpec{
			Rel:     sqlgraph.M2O,
			Inverse: true,
			Table:   attestation.AttestationCollectionTable,
			Columns: []string{attestation.AttestationCollectionColumn},
			Bidi:    false,
			Target: &sqlgraph.EdgeTarget{
				IDSpec: &sqlgraph.FieldSpec{
					Type:   field.TypeInt,
					Column: attestationcollection.FieldID,
				},
			},
		}
		for _, k := range nodes {
			edge.Target.Nodes = append(edge.Target.Nodes, k)
		}
		_spec.Edges.Add = append(_spec.Edges.Add, edge)
	}
	if n, err = sqlgraph.UpdateNodes(ctx, au.driver, _spec); err != nil {
		if _, ok := err.(*sqlgraph.NotFoundError); ok {
			err = &NotFoundError{attestation.Label}
		} else if sqlgraph.IsConstraintError(err) {
			err = &ConstraintError{msg: err.Error(), wrap: err}
		}
		return 0, err
	}
	return n, nil
}

// AttestationUpdateOne is the builder for updating a single Attestation entity.
type AttestationUpdateOne struct {
	config
	fields   []string
	hooks    []Hook
	mutation *AttestationMutation
}

// SetType sets the "type" field.
func (auo *AttestationUpdateOne) SetType(s string) *AttestationUpdateOne {
	auo.mutation.SetType(s)
	return auo
}

// SetAttestationCollectionID sets the "attestation_collection" edge to the AttestationCollection entity by ID.
func (auo *AttestationUpdateOne) SetAttestationCollectionID(id int) *AttestationUpdateOne {
	auo.mutation.SetAttestationCollectionID(id)
	return auo
}

// SetAttestationCollection sets the "attestation_collection" edge to the AttestationCollection entity.
func (auo *AttestationUpdateOne) SetAttestationCollection(a *AttestationCollection) *AttestationUpdateOne {
	return auo.SetAttestationCollectionID(a.ID)
}

// Mutation returns the AttestationMutation object of the builder.
func (auo *AttestationUpdateOne) Mutation() *AttestationMutation {
	return auo.mutation
}

// ClearAttestationCollection clears the "attestation_collection" edge to the AttestationCollection entity.
func (auo *AttestationUpdateOne) ClearAttestationCollection() *AttestationUpdateOne {
	auo.mutation.ClearAttestationCollection()
	return auo
}

// Select allows selecting one or more fields (columns) of the returned entity.
// The default is selecting all fields defined in the entity schema.
func (auo *AttestationUpdateOne) Select(field string, fields ...string) *AttestationUpdateOne {
	auo.fields = append([]string{field}, fields...)
	return auo
}

// Save executes the query and returns the updated Attestation entity.
func (auo *AttestationUpdateOne) Save(ctx context.Context) (*Attestation, error) {
	var (
		err  error
		node *Attestation
	)
	if len(auo.hooks) == 0 {
		if err = auo.check(); err != nil {
			return nil, err
		}
		node, err = auo.sqlSave(ctx)
	} else {
		var mut Mutator = MutateFunc(func(ctx context.Context, m Mutation) (Value, error) {
			mutation, ok := m.(*AttestationMutation)
			if !ok {
				return nil, fmt.Errorf("unexpected mutation type %T", m)
			}
			if err = auo.check(); err != nil {
				return nil, err
			}
			auo.mutation = mutation
			node, err = auo.sqlSave(ctx)
			mutation.done = true
			return node, err
		})
		for i := len(auo.hooks) - 1; i >= 0; i-- {
			if auo.hooks[i] == nil {
				return nil, fmt.Errorf("ent: uninitialized hook (forgotten import ent/runtime?)")
			}
			mut = auo.hooks[i](mut)
		}
		v, err := mut.Mutate(ctx, auo.mutation)
		if err != nil {
			return nil, err
		}
		nv, ok := v.(*Attestation)
		if !ok {
			return nil, fmt.Errorf("unexpected node type %T returned from AttestationMutation", v)
		}
		node = nv
	}
	return node, err
}

// SaveX is like Save, but panics if an error occurs.
func (auo *AttestationUpdateOne) SaveX(ctx context.Context) *Attestation {
	node, err := auo.Save(ctx)
	if err != nil {
		panic(err)
	}
	return node
}

// Exec executes the query on the entity.
func (auo *AttestationUpdateOne) Exec(ctx context.Context) error {
	_, err := auo.Save(ctx)
	return err
}

// ExecX is like Exec, but panics if an error occurs.
func (auo *AttestationUpdateOne) ExecX(ctx context.Context) {
	if err := auo.Exec(ctx); err != nil {
		panic(err)
	}
}

// check runs all checks and user-defined validators on the builder.
func (auo *AttestationUpdateOne) check() error {
	if v, ok := auo.mutation.GetType(); ok {
		if err := attestation.TypeValidator(v); err != nil {
			return &ValidationError{Name: "type", err: fmt.Errorf(`ent: validator failed for field "Attestation.type": %w`, err)}
		}
	}
	if _, ok := auo.mutation.AttestationCollectionID(); auo.mutation.AttestationCollectionCleared() && !ok {
		return errors.New(`ent: clearing a required unique edge "Attestation.attestation_collection"`)
	}
	return nil
}

func (auo *AttestationUpdateOne) sqlSave(ctx context.Context) (_node *Attestation, err error) {
	_spec := &sqlgraph.UpdateSpec{
		Node: &sqlgraph.NodeSpec{
			Table:   attestation.Table,
			Columns: attestation.Columns,
			ID: &sqlgraph.FieldSpec{
				Type:   field.TypeInt,
				Column: attestation.FieldID,
			},
		},
	}
	id, ok := auo.mutation.ID()
	if !ok {
		return nil, &ValidationError{Name: "id", err: errors.New(`ent: missing "Attestation.id" for update`)}
	}
	_spec.Node.ID.Value = id
	if fields := auo.fields; len(fields) > 0 {
		_spec.Node.Columns = make([]string, 0, len(fields))
		_spec.Node.Columns = append(_spec.Node.Columns, attestation.FieldID)
		for _, f := range fields {
			if !attestation.ValidColumn(f) {
				return nil, &ValidationError{Name: f, err: fmt.Errorf("ent: invalid field %q for query", f)}
			}
			if f != attestation.FieldID {
				_spec.Node.Columns = append(_spec.Node.Columns, f)
			}
		}
	}
	if ps := auo.mutation.predicates; len(ps) > 0 {
		_spec.Predicate = func(selector *sql.Selector) {
			for i := range ps {
				ps[i](selector)
			}
		}
	}
	if value, ok := auo.mutation.GetType(); ok {
		_spec.Fields.Set = append(_spec.Fields.Set, &sqlgraph.FieldSpec{
			Type:   field.TypeString,
			Value:  value,
			Column: attestation.FieldType,
		})
	}
	if auo.mutation.AttestationCollectionCleared() {
		edge := &sqlgraph.EdgeSpec{
			Rel:     sqlgraph.M2O,
			Inverse: true,
			Table:   attestation.AttestationCollectionTable,
			Columns: []string{attestation.AttestationCollectionColumn},
			Bidi:    false,
			Target: &sqlgraph.EdgeTarget{
				IDSpec: &sqlgraph.FieldSpec{
					Type:   field.TypeInt,
					Column: attestationcollection.FieldID,
				},
			},
		}
		_spec.Edges.Clear = append(_spec.Edges.Clear, edge)
	}
	if nodes := auo.mutation.AttestationCollectionIDs(); len(nodes) > 0 {
		edge := &sqlgraph.EdgeSpec{
			Rel:     sqlgraph.M2O,
			Inverse: true,
			Table:   attestation.AttestationCollectionTable,
			Columns: []string{attestation.AttestationCollectionColumn},
			Bidi:    false,
			Target: &sqlgraph.EdgeTarget{
				IDSpec: &sqlgraph.FieldSpec{
					Type:   field.TypeInt,
					Column: attestationcollection.FieldID,
				},
			},
		}
		for _, k := range nodes {
			edge.Target.Nodes = append(edge.Target.Nodes, k)
		}
		_spec.Edges.Add = append(_spec.Edges.Add, edge)
	}
	_node = &Attestation{config: auo.config}
	_spec.Assign = _node.assignValues
	_spec.ScanValues = _node.scanValues
	if err = sqlgraph.UpdateNode(ctx, auo.driver, _spec); err != nil {
		if _, ok := err.(*sqlgraph.NotFoundError); ok {
			err = &NotFoundError{attestation.Label}
		} else if sqlgraph.IsConstraintError(err) {
			err = &ConstraintError{msg: err.Error(), wrap: err}
		}
		return nil, err
	}
	return _node, nil
}
